"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _objectWithoutPropertiesLoose2 = _interopRequireDefault(require("@babel/runtime/helpers/objectWithoutPropertiesLoose"));

const _require = require(`../constants`),
      Actions = _require.Actions;

module.exports = (state = {
  messages: [],
  activities: {},
  status: ``
}, action) => {
  switch (action.type) {
    case Actions.SetStatus:
      {
        return Object.assign({}, state, {
          status: action.payload
        });
      }

    case Actions.Log:
      {
        if (!action.payload.text) {
          // set empty character to fix ink
          action.payload.text = `\u2800`;
        }

        return Object.assign({}, state, {
          messages: [...state.messages, action.payload]
        });
      }

    case Actions.StartActivity:
      {
        const id = action.payload.id;
        return Object.assign({}, state, {
          activities: Object.assign({}, state.activities, {
            [id]: action.payload
          })
        });
      }

    case Actions.UpdateActivity:
    case Actions.PendingActivity:
      {
        const _action$payload = action.payload,
              id = _action$payload.id,
              rest = (0, _objectWithoutPropertiesLoose2.default)(_action$payload, ["id"]);
        const activity = state.activities[id];
        return Object.assign({}, state, {
          activities: Object.assign({}, state.activities, {
            [id]: Object.assign({}, activity, {}, rest)
          })
        });
      }

    case Actions.ActivityErrored:
      {
        const id = action.payload.id;
        const activity = state.activities[id];
        return Object.assign({}, state, {
          activities: Object.assign({}, state.activities, {
            [id]: Object.assign({}, activity, {
              errored: true
            })
          })
        });
      }

    case Actions.EndActivity:
    case Actions.CancelActivity:
      {
        const _action$payload2 = action.payload,
              id = _action$payload2.id,
              status = _action$payload2.status,
              duration = _action$payload2.duration;
        const activity = state.activities[id];

        if (!activity) {
          return state;
        }

        const activities = Object.assign({}, state.activities);
        activities[id] = Object.assign({}, activity, {
          status,
          duration
        });
        return Object.assign({}, state, {
          activities
        });
      }

    case Actions.SetLogs:
      {
        return action.payload;
      }
  }

  return state;
};